@@ -2,7 +2,7 @@
 unittest2
 
 unittest2 is a backport of the new features added to the unittest testing
-framework in Python 2.7. It is tested to run on Python 2.4 - 2.6.
+framework in Python 2.7 and beyond. It is tested to run on Python 2.4 - 2.7.
 
 To use unittest2 instead of unittest simply replace ``import unittest`` with
 ``import unittest2``.
@@ -29,14 +29,15 @@ SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 __all__ = ['TestResult', 'TestCase', 'TestSuite',
            'TextTestRunner', 'TestLoader', 'FunctionTestCase', 'main',
            'defaultTestLoader', 'SkipTest', 'skip', 'skipIf', 'skipUnless',
-           'expectedFailure', 'TextTestResult', '__version__']
+           'expectedFailure', 'TextTestResult', '__version__', 'collector']
 
-__version__ = '0.4.2'
+__version__ = '0.6.0'
 
 # Expose obsolete functions for backwards compatibility
 __all__.extend(['getTestCaseNames', 'makeSuite', 'findTestCases'])
 
 
+from unittest2.collector import collector
 from unittest2.result import TestResult
 from unittest2.case import (
     TestCase, FunctionTestCase, SkipTest, skip, skipIf,
@@ -58,10 +59,10 @@ except ImportError:
     # Compatibility with platforms that don't have the signal module
     pass
 else:
-    __all__.extend(['installHandler', 'registerResult', 'removeResult', 
+    __all__.extend(['installHandler', 'registerResult', 'removeResult',
                     'removeHandler'])
 
 # deprecated
 _TextTestResult = TextTestResult
 
-__unittest = True
\ No newline at end of file
+__unittest = True
