@@ -1,8 +1,12 @@
+from copy import deepcopy
 import difflib
+import pickle
 import pprint
 import re
+import sys
 
-from copy import deepcopy
+import six
+from six import b, u
 
 import unittest2
 
@@ -42,112 +46,6 @@ class Test(object):
             self.events.append('tearDown')
 
 
-
-class TestCleanUp(unittest2.TestCase):
-
-    def testCleanUp(self):
-        class TestableTest(unittest2.TestCase):
-            def testNothing(self):
-                pass
-
-        test = TestableTest('testNothing')
-        self.assertEqual(test._cleanups, [])
-
-        cleanups = []
-
-        def cleanup1(*args, **kwargs):
-            cleanups.append((1, args, kwargs))
-
-        def cleanup2(*args, **kwargs):
-            cleanups.append((2, args, kwargs))
-
-        test.addCleanup(cleanup1, 1, 2, 3, four='hello', five='goodbye')
-        test.addCleanup(cleanup2)
-
-        self.assertEqual(test._cleanups,
-                         [(cleanup1, (1, 2, 3), dict(four='hello', five='goodbye')),
-                          (cleanup2, (), {})])
-
-        result = test.doCleanups()
-        self.assertTrue(result)
-
-        self.assertEqual(cleanups, [(2, (), {}), (1, (1, 2, 3), dict(four='hello', five='goodbye'))])
-
-    def testCleanUpWithErrors(self):
-        class TestableTest(unittest2.TestCase):
-            def testNothing(self):
-                pass
-
-        class MockResult(object):
-            errors = []
-            def addError(self, test, exc_info):
-                self.errors.append((test, exc_info))
-
-        result = MockResult()
-        test = TestableTest('testNothing')
-        test._resultForDoCleanups = result
-
-        exc1 = Exception('foo')
-        exc2 = Exception('bar')
-        def cleanup1():
-            raise exc1
-
-        def cleanup2():
-            raise exc2
-
-        test.addCleanup(cleanup1)
-        test.addCleanup(cleanup2)
-
-        self.assertFalse(test.doCleanups())
-
-        (test1, (Type1, instance1, _)), (test2, (Type2, instance2, _)) = reversed(MockResult.errors)
-        self.assertEqual((test1, Type1, instance1), (test, Exception, exc1))
-        self.assertEqual((test2, Type2, instance2), (test, Exception, exc2))
-
-    def testCleanupInRun(self):
-        blowUp = False
-        ordering = []
-
-        class TestableTest(unittest2.TestCase):
-            def setUp(self):
-                ordering.append('setUp')
-                if blowUp:
-                    raise Exception('foo')
-
-            def testNothing(self):
-                ordering.append('test')
-
-            def tearDown(self):
-                ordering.append('tearDown')
-
-        test = TestableTest('testNothing')
-
-        def cleanup1():
-            ordering.append('cleanup1')
-        def cleanup2():
-            ordering.append('cleanup2')
-        test.addCleanup(cleanup1)
-        test.addCleanup(cleanup2)
-
-        def success(some_test):
-            self.assertEqual(some_test, test)
-            ordering.append('success')
-
-        result = unittest2.TestResult()
-        result.addSuccess = success
-
-        test.run(result)
-        self.assertEqual(ordering, ['setUp', 'test', 'tearDown',
-                                    'cleanup2', 'cleanup1', 'success'])
-
-        blowUp = True
-        ordering = []
-        test = TestableTest('testNothing')
-        test.addCleanup(cleanup1)
-        test.run(result)
-        self.assertEqual(ordering, ['setUp', 'cleanup1'])
-
-
 class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
 
     ### Set up attributes used by inherited tests
@@ -281,7 +179,7 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
                 super(Foo, self).test()
                 raise RuntimeError('raised by Foo.test')
 
-        expected = ['startTest', 'setUp', 'test', 'addError', 'tearDown',
+        expected = ['startTest', 'setUp', 'test', 'tearDown', 'addError',
                     'stopTest']
         Foo(events).run(result)
         self.assertEqual(events, expected)
@@ -299,8 +197,8 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
                 super(Foo, self).test()
                 raise RuntimeError('raised by Foo.test')
 
-        expected = ['startTestRun', 'startTest', 'setUp', 'test', 'addError',
-                    'tearDown', 'stopTest', 'stopTestRun']
+        expected = ['startTestRun', 'startTest', 'setUp', 'test',
+                    'tearDown', 'addError', 'stopTest', 'stopTestRun']
         Foo(events).run()
         self.assertEqual(events, expected)
 
@@ -320,7 +218,7 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
                 super(Foo, self).test()
                 self.fail('raised by Foo.test')
 
-        expected = ['startTest', 'setUp', 'test', 'addFailure', 'tearDown',
+        expected = ['startTest', 'setUp', 'test', 'tearDown', 'addFailure',
                     'stopTest']
         Foo(events).run(result)
         self.assertEqual(events, expected)
@@ -335,8 +233,8 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
                 super(Foo, self).test()
                 self.fail('raised by Foo.test')
 
-        expected = ['startTestRun', 'startTest', 'setUp', 'test', 'addFailure',
-                    'tearDown', 'stopTest', 'stopTestRun']
+        expected = ['startTestRun', 'startTest', 'setUp', 'test',
+                    'tearDown', 'addFailure', 'stopTest', 'stopTestRun']
         events = []
         Foo(events).run()
         self.assertEqual(events, expected)
@@ -477,7 +375,7 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
             def runTest(self):
                 pass
 
-        self.assertIsInstance(Foo().id(), basestring)
+        self.assertIsInstance(Foo().id(), six.string_types)
 
     # "If result is omitted or None, a temporary result object is created
     # and used, but is not made available to the caller. As TestCase owns the
@@ -599,7 +497,7 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
         self.assertRaises(unittest2.TestCase.failureException,
                           self.assertDictContainsSubset, {'a': 1, 'c': 1},
                           {'a': 1}, '.*Missing:.*Mismatched values:.*')
-        
+
         self.assertRaises(self.failureException,
                           self.assertDictContainsSubset, {1: "one"}, {})
 
@@ -708,7 +606,7 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
         self.assertItemsEqual([{'a': 1}, {'b': 2}], [{'b': 2}, {'a': 1}])
         self.assertRaises(self.failureException, self.assertItemsEqual,
                           [[1]], [[2]])
-        
+
         # Test unsortable objects
         self.assertItemsEqual([2j, None], [None, 2j])
         self.assertRaises(self.failureException, self.assertItemsEqual,
@@ -798,61 +696,61 @@ class Test_TestCase(unittest2.TestCase, EqualityMixin, HashingMixin):
         self.assertRaises(self.failureException, self.assertLessEqual, 'bug', 'ant')
 
         # Try Unicode
-        self.assertGreater(u'bug', u'ant')
-        self.assertGreaterEqual(u'bug', u'ant')
-        self.assertGreaterEqual(u'ant', u'ant')
-        self.assertLess(u'ant', u'bug')
-        self.assertLessEqual(u'ant', u'bug')
-        self.assertLessEqual(u'ant', u'ant')
-        self.assertRaises(self.failureException, self.assertGreater, u'ant', u'bug')
-        self.assertRaises(self.failureException, self.assertGreater, u'ant', u'ant')
-        self.assertRaises(self.failureException, self.assertGreaterEqual, u'ant',
-                          u'bug')
-        self.assertRaises(self.failureException, self.assertLess, u'bug', u'ant')
-        self.assertRaises(self.failureException, self.assertLess, u'ant', u'ant')
-        self.assertRaises(self.failureException, self.assertLessEqual, u'bug', u'ant')
+        self.assertGreater(u('bug'), u('ant'))
+        self.assertGreaterEqual(u('bug'), u('ant'))
+        self.assertGreaterEqual(u('ant'), u('ant'))
+        self.assertLess(u('ant'), u('bug'))
+        self.assertLessEqual(u('ant'), u('bug'))
+        self.assertLessEqual(u('ant'), u('ant'))
+        self.assertRaises(self.failureException, self.assertGreater, u('ant'), u('bug'))
+        self.assertRaises(self.failureException, self.assertGreater, u('ant'), u('ant'))
+        self.assertRaises(self.failureException, self.assertGreaterEqual, u('ant'),
+                          u('bug'))
+        self.assertRaises(self.failureException, self.assertLess, u('bug'), u('ant'))
+        self.assertRaises(self.failureException, self.assertLess, u('ant'), u('ant'))
+        self.assertRaises(self.failureException, self.assertLessEqual, u('bug'), u('ant'))
 
         # Try Mixed String/Unicode
-        self.assertGreater('bug', u'ant')
-        self.assertGreater(u'bug', 'ant')
-        self.assertGreaterEqual('bug', u'ant')
-        self.assertGreaterEqual(u'bug', 'ant')
-        self.assertGreaterEqual('ant', u'ant')
-        self.assertGreaterEqual(u'ant', 'ant')
-        self.assertLess('ant', u'bug')
-        self.assertLess(u'ant', 'bug')
-        self.assertLessEqual('ant', u'bug')
-        self.assertLessEqual(u'ant', 'bug')
-        self.assertLessEqual('ant', u'ant')
-        self.assertLessEqual(u'ant', 'ant')
-        self.assertRaises(self.failureException, self.assertGreater, 'ant', u'bug')
-        self.assertRaises(self.failureException, self.assertGreater, u'ant', 'bug')
-        self.assertRaises(self.failureException, self.assertGreater, 'ant', u'ant')
-        self.assertRaises(self.failureException, self.assertGreater, u'ant', 'ant')
+        self.assertGreater('bug', u('ant'))
+        self.assertGreater(u('bug'), 'ant')
+        self.assertGreaterEqual('bug', u('ant'))
+        self.assertGreaterEqual(u('bug'), 'ant')
+        self.assertGreaterEqual('ant', u('ant'))
+        self.assertGreaterEqual(u('ant'), 'ant')
+        self.assertLess('ant', u('bug'))
+        self.assertLess(u('ant'), 'bug')
+        self.assertLessEqual('ant', u('bug'))
+        self.assertLessEqual(u('ant'), 'bug')
+        self.assertLessEqual('ant', u('ant'))
+        self.assertLessEqual(u('ant'), 'ant')
+        self.assertRaises(self.failureException, self.assertGreater, 'ant', u('bug'))
+        self.assertRaises(self.failureException, self.assertGreater, u('ant'), 'bug')
+        self.assertRaises(self.failureException, self.assertGreater, 'ant', u('ant'))
+        self.assertRaises(self.failureException, self.assertGreater, u('ant'), 'ant')
         self.assertRaises(self.failureException, self.assertGreaterEqual, 'ant',
-                          u'bug')
-        self.assertRaises(self.failureException, self.assertGreaterEqual, u'ant',
+                          u('bug'))
+        self.assertRaises(self.failureException, self.assertGreaterEqual, u('ant'),
                           'bug')
-        self.assertRaises(self.failureException, self.assertLess, 'bug', u'ant')
-        self.assertRaises(self.failureException, self.assertLess, u'bug', 'ant')
-        self.assertRaises(self.failureException, self.assertLess, 'ant', u'ant')
-        self.assertRaises(self.failureException, self.assertLess, u'ant', 'ant')
-        self.assertRaises(self.failureException, self.assertLessEqual, 'bug', u'ant')
-        self.assertRaises(self.failureException, self.assertLessEqual, u'bug', 'ant')
+        self.assertRaises(self.failureException, self.assertLess, 'bug', u('ant'))
+        self.assertRaises(self.failureException, self.assertLess, u('bug'), 'ant')
+        self.assertRaises(self.failureException, self.assertLess, 'ant', u('ant'))
+        self.assertRaises(self.failureException, self.assertLess, u('ant'), 'ant')
+        self.assertRaises(self.failureException, self.assertLessEqual, 'bug', u('ant'))
+        self.assertRaises(self.failureException, self.assertLessEqual, u('bug'), 'ant')
 
     def testAssertMultiLineEqual(self):
-        sample_text = """\
+        sample_text = u("""\
 http://www.python.org/doc/2.3/lib/module-unittest.html
 test case
     A test case is the smallest unit of testing. [...]
-"""
-        revised_sample_text = """\
+""")
+        revised_sample_text = u("""\
 http://www.python.org/doc/2.4.1/lib/module-unittest.html
 test case
     A test case is the smallest unit of testing. [...] You may provide your
     own implementation that does not subclass from TestCase, of course.
-"""
-        sample_text_error = """\
+""")
+        sample_text_error = u("""\
 - http://www.python.org/doc/2.3/lib/module-unittest.html
 ?                             ^
 + http://www.python.org/doc/2.4.1/lib/module-unittest.html
@@ -862,19 +760,33 @@ test case
 +     A test case is the smallest unit of testing. [...] You may provide your
 ?                                                       +++++++++++++++++++++
 +     own implementation that does not subclass from TestCase, of course.
-"""
+""")
         self.maxDiff = None
-        for type_changer in (lambda x: x, lambda x: x.decode('utf8')):
+        # On python 3 we skip bytestrings as they fail the string
+        # check. in assertMultiLineEqual
+        changers = [lambda x: x]
+        if sys.version_info[0] < 3:
+            changers.append(lambda x: x.encode('utf8'))
+        for type_changer in changers:
             try:
                 self.assertMultiLineEqual(type_changer(sample_text),
                                           type_changer(revised_sample_text))
-            except self.failureException, e:
+            except self.failureException:
+                e = sys.exc_info()[1]
                 # need to remove the first line of the error message
-                error = str(e).encode('utf8').split('\n', 1)[1]
+                error_str = str(e)
+                if not isinstance(error_str, six.text_type):
+                    error_str = error_str.decode('utf8')
+                error_lines = error_str.split(u('\n'), 1)
+                if len(error_lines) > 1:
+                    error = error_lines[1]
+                else:
+                    error = error_lines[0]
 
                 # assertMultiLineEqual is hooked up as the default for
                 # unicode strings - so we can't use it for this check
-                self.assertTrue(sample_text_error == error)
+                self.assertTrue(sample_text_error == error,
+                    "error %r not in sample_text %r" % (error_str, sample_text_error))
 
     def testAssertSequenceEqualMaxDiff(self):
         self.assertEqual(self.maxDiff, 80*8)
@@ -888,7 +800,8 @@ test case
         self.maxDiff = len(diff)//2
         try:
             self.assertSequenceEqual(seq1, seq2)
-        except self.failureException, e:
+        except self.failureException:
+            e = sys.exc_info()[1]
             msg = e.args[0]
         else:
             self.fail('assertSequenceEqual did not fail.')
@@ -898,7 +811,8 @@ test case
         self.maxDiff = len(diff) * 2
         try:
             self.assertSequenceEqual(seq1, seq2)
-        except self.failureException, e:
+        except self.failureException:
+            e = sys.exc_info()[1]
             msg = e.args[0]
         else:
             self.fail('assertSequenceEqual did not fail.')
@@ -908,7 +822,8 @@ test case
         self.maxDiff = None
         try:
             self.assertSequenceEqual(seq1, seq2)
-        except self.failureException, e:
+        except self.failureException:
+            e = sys.exc_info()[1]
             msg = e.args[0]
         else:
             self.fail('assertSequenceEqual did not fail.')
@@ -936,7 +851,8 @@ test case
         test._truncateMessage = truncate
         try:
             test.assertDictEqual({}, {1: 0})
-        except self.failureException, e:
+        except self.failureException:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), 'foo')
         else:
             self.fail('assertDictEqual did not fail')
@@ -948,7 +864,8 @@ test case
         test._truncateMessage = truncate
         try:
             test.assertMultiLineEqual('foo', 'bar')
-        except self.failureException, e:
+        except self.failureException:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), 'foo')
         else:
             self.fail('assertMultiLineEqual did not fail')
@@ -959,71 +876,57 @@ test case
         self.assertIsNotNone('DjZoPloGears on Rails')
         self.assertRaises(self.failureException, self.assertIsNotNone, None)
 
-    def testAssertRegexpMatches(self):
-        self.assertRegexpMatches('asdfabasdf', r'ab+')
-        self.assertRaises(self.failureException, self.assertRegexpMatches,
+    def testAssertRegex(self):
+        self.assertRegex('asdfabasdf', r'ab+')
+        self.assertRaises(self.failureException, self.assertRegex,
                           'saaas', r'aaaa')
 
-    def testAssertRaisesRegexp(self):
+    def testAssertRaisesRegex(self):
         class ExceptionMock(Exception):
             pass
 
         def Stub():
             raise ExceptionMock('We expect')
 
-        self.assertRaisesRegexp(ExceptionMock, re.compile('expect$'), Stub)
-        self.assertRaisesRegexp(ExceptionMock, 'expect$', Stub)
-        self.assertRaisesRegexp(ExceptionMock, u'expect$', Stub)
+        self.assertRaisesRegex(ExceptionMock, re.compile('expect$'), Stub)
+        self.assertRaisesRegex(ExceptionMock, 'expect$', Stub)
+        self.assertRaisesRegex(ExceptionMock, u('expect$'), Stub)
 
-    def testAssertNotRaisesRegexp(self):
-        self.assertRaisesRegexp(
+    def testAssertNotRaisesRegex(self):
+        self.assertRaisesRegex(
                 self.failureException, '^Exception not raised$',
-                self.assertRaisesRegexp, Exception, re.compile('x'),
+                self.assertRaisesRegex, Exception, re.compile('x'),
                 lambda: None)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
                 self.failureException, '^Exception not raised$',
-                self.assertRaisesRegexp, Exception, 'x',
+                self.assertRaisesRegex, Exception, 'x',
                 lambda: None)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
                 self.failureException, '^Exception not raised$',
-                self.assertRaisesRegexp, Exception, u'x',
+                self.assertRaisesRegex, Exception, u('x'),
                 lambda: None)
 
-    def testAssertRaisesRegexpMismatch(self):
+    def testAssertRaisesRegexMismatch(self):
         def Stub():
             raise Exception('Unexpected')
 
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
                 self.failureException,
                 r'"\^Expected\$" does not match "Unexpected"',
-                self.assertRaisesRegexp, Exception, '^Expected$',
+                self.assertRaisesRegex, Exception, '^Expected$',
                 Stub)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
                 self.failureException,
                 r'"\^Expected\$" does not match "Unexpected"',
-                self.assertRaisesRegexp, Exception, u'^Expected$',
+                self.assertRaisesRegex, Exception, u('^Expected$'),
                 Stub)
-        self.assertRaisesRegexp(
+        self.assertRaisesRegex(
                 self.failureException,
                 r'"\^Expected\$" does not match "Unexpected"',
-                self.assertRaisesRegexp, Exception,
+                self.assertRaisesRegex, Exception,
                 re.compile('^Expected$'), Stub)
 
 
-    def testSynonymAssertMethodNames(self):
-        """Test undocumented method name synonyms.
-
-        Please do not use these methods names in your own code.
-
-        This test confirms their continued existence and functionality
-        in order to avoid breaking existing code.
-        """
-        self.assertNotEquals(3, 5)
-        self.assertEquals(3, 3)
-        self.assertAlmostEquals(2.0, 2.0)
-        self.assertNotAlmostEquals(3.0, 5.0)
-        self.assert_(True)
-
     def testDeepcopy(self):
         # Issue: 5660
         class TestableTest(unittest2.TestCase):
@@ -1036,5 +939,103 @@ test case
         deepcopy(test)
 
 
+    def testPickle(self):
+        # Issue 10326
+
+        # Can't use TestCase classes defined in Test class as
+        # pickle does not work with inner classes
+        test = unittest2.TestCase('run')
+        for protocol in range(pickle.HIGHEST_PROTOCOL + 1):
+
+            # blew up prior to fix
+            pickled_test = pickle.dumps(test, protocol=protocol)
+            unpickled_test = pickle.loads(pickled_test)
+            self.assertEqual(test, unpickled_test)
+
+            # exercise the TestCase instance in a way that will invoke
+            # the type equality lookup mechanism
+            unpickled_test.assertEqual(set(), set())
+
+
+    def testKeyboardInterrupt(self):
+        def _raise(self=None):
+            raise KeyboardInterrupt
+        def nothing(self):
+            pass
+
+        class Test1(unittest2.TestCase):
+            test_something = _raise
+
+        class Test2(unittest2.TestCase):
+            setUp = _raise
+            test_something = nothing
+
+        class Test3(unittest2.TestCase):
+            test_something = nothing
+            tearDown = _raise
+
+        class Test4(unittest2.TestCase):
+            def test_something(self):
+                self.addCleanup(_raise)
+
+        for klass in (Test1, Test2, Test3, Test4):
+            self.assertRaises(KeyboardInterrupt,
+                klass('test_something').run)
+
+    def testSkippingEverywhere(self):
+        def _skip(self=None):
+            raise unittest2.SkipTest('some reason')
+        def nothing(self):
+            pass
+
+        class Test1(unittest2.TestCase):
+            test_something = _skip
+
+        class Test2(unittest2.TestCase):
+            setUp = _skip
+            test_something = nothing
+
+        class Test3(unittest2.TestCase):
+            test_something = nothing
+            tearDown = _skip
+
+        class Test4(unittest2.TestCase):
+            def test_something(self):
+                self.addCleanup(_skip)
+
+        for klass in (Test1, Test2, Test3, Test4):
+            result = unittest2.TestResult()
+            klass('test_something').run(result)
+            self.assertEqual(len(result.skipped), 1)
+            self.assertEqual(result.testsRun, 1)
+
+    def testSystemExit(self):
+        def _raise(self=None):
+            raise SystemExit
+        def nothing(self):
+            pass
+
+        class Test1(unittest2.TestCase):
+            test_something = _raise
+
+        class Test2(unittest2.TestCase):
+            setUp = _raise
+            test_something = nothing
+
+        class Test3(unittest2.TestCase):
+            test_something = nothing
+            tearDown = _raise
+
+        class Test4(unittest2.TestCase):
+            def test_something(self):
+                self.addCleanup(_raise)
+
+        for klass in (Test1, Test2, Test3, Test4):
+            result = unittest2.TestResult()
+            klass('test_something').run(result)
+            self.assertEqual(len(result.errors), 1)
+            self.assertEqual(result.testsRun, 1)
+
+
 if __name__ == "__main__":
     unittest2.main()
