@@ -2,6 +2,7 @@ import sys
 import types
 
 import unittest2
+from unittest2 import util
 
 
 class Test_TestLoader(unittest2.TestCase):
@@ -166,11 +167,11 @@ class Test_TestLoader(unittest2.TestCase):
         loader = unittest2.TestLoader()
         suite = loader.loadTestsFromModule(m)
         self.assertIsInstance(suite, unittest2.TestSuite)
-        self.assertEquals(load_tests_args, [loader, suite, None])
+        self.assertEqual(load_tests_args, [loader, suite, None])
 
         load_tests_args = []
         suite = loader.loadTestsFromModule(m, use_load_tests=False)
-        self.assertEquals(load_tests_args, [])
+        self.assertEqual(load_tests_args, [])
 
     def test_loadTestsFromModule__faulty_load_tests(self):
         m = types.ModuleType('m')
@@ -184,9 +185,9 @@ class Test_TestLoader(unittest2.TestCase):
         self.assertIsInstance(suite, unittest2.TestSuite)
         self.assertEqual(suite.countTestCases(), 1)
         test = list(suite)[0]
-        
-        self.assertRaisesRegexp(TypeError, "some failure", test.m)
-        
+
+        self.assertRaisesRegex(TypeError, "some failure", test.m)
+
 
     ################################################################
     ### /Tests for TestLoader.loadTestsFromModule()
@@ -205,7 +206,8 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromName('')
-        except ValueError, e:
+        except ValueError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "Empty module name")
         else:
             self.fail("TestLoader.loadTestsFromName failed to raise ValueError")
@@ -238,8 +240,9 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromName('sdasfasfasdf')
-        except ImportError, e:
-            self.assertEqual(str(e), "No module named sdasfasfasdf")
+        except ImportError:
+            e = sys.exc_info()[1]
+            self.assertRegex(str(e), "No module named '?sdasfasfasdf'?")
         else:
             self.fail("TestLoader.loadTestsFromName failed to raise ImportError")
 
@@ -254,7 +257,8 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromName('unittest2.sdasfasfasdf')
-        except AttributeError, e:
+        except AttributeError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "'module' object has no attribute 'sdasfasfasdf'")
         else:
             self.fail("TestLoader.loadTestsFromName failed to raise AttributeError")
@@ -271,7 +275,8 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromName('sdasfasfasdf', unittest2)
-        except AttributeError, e:
+        except AttributeError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "'module' object has no attribute 'sdasfasfasdf'")
         else:
             self.fail("TestLoader.loadTestsFromName failed to raise AttributeError")
@@ -425,7 +430,8 @@ class Test_TestLoader(unittest2.TestCase):
         loader = unittest2.TestLoader()
         try:
             loader.loadTestsFromName('testcase_1.testfoo', m)
-        except AttributeError, e:
+        except AttributeError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "type object 'MyTestCase' has no attribute 'testfoo'")
         else:
             self.fail("Failed to raise AttributeError")
@@ -524,12 +530,8 @@ class Test_TestLoader(unittest2.TestCase):
         # We're going to try to load this module as a side-effect, so it
         # better not be loaded before we try.
         #
-        # Why pick audioop? Google shows it isn't used very often, so there's
-        # a good chance that it won't be imported when this test is run
-        module_name = 'audioop'
-
-        if module_name in sys.modules:
-            del sys.modules[module_name]
+        module_name = 'unittest2.test.dummy'
+        sys.modules.pop(module_name, None)
 
         loader = unittest2.TestLoader()
         try:
@@ -538,7 +540,7 @@ class Test_TestLoader(unittest2.TestCase):
             self.assertIsInstance(suite, loader.suiteClass)
             self.assertEqual(list(suite), [])
 
-            # audioop should now be loaded, thanks to loadTestsFromName()
+            # module should now be loaded, thanks to loadTestsFromName()
             self.assertIn(module_name, sys.modules)
         finally:
             if module_name in sys.modules:
@@ -587,7 +589,8 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromNames([''])
-        except ValueError, e:
+        except ValueError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "Empty module name")
         else:
             self.fail("TestLoader.loadTestsFromNames failed to raise ValueError")
@@ -622,8 +625,9 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromNames(['sdasfasfasdf'])
-        except ImportError, e:
-            self.assertEqual(str(e), "No module named sdasfasfasdf")
+        except ImportError:
+            e = sys.exc_info()[1]
+            self.assertRegex(str(e), "No module named '?sdasfasfasdf'?")
         else:
             self.fail("TestLoader.loadTestsFromNames failed to raise ImportError")
 
@@ -638,7 +642,8 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromNames(['unittest2.sdasfasfasdf', 'unittest2'])
-        except AttributeError, e:
+        except AttributeError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "'module' object has no attribute 'sdasfasfasdf'")
         else:
             self.fail("TestLoader.loadTestsFromNames failed to raise AttributeError")
@@ -657,7 +662,8 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromNames(['sdasfasfasdf'], unittest2)
-        except AttributeError, e:
+        except AttributeError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "'module' object has no attribute 'sdasfasfasdf'")
         else:
             self.fail("TestLoader.loadTestsFromName failed to raise AttributeError")
@@ -676,7 +682,8 @@ class Test_TestLoader(unittest2.TestCase):
 
         try:
             loader.loadTestsFromNames(['TestCase', 'sdasfasfasdf'], unittest2)
-        except AttributeError, e:
+        except AttributeError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "'module' object has no attribute 'sdasfasfasdf'")
         else:
             self.fail("TestLoader.loadTestsFromName failed to raise AttributeError")
@@ -825,7 +832,8 @@ class Test_TestLoader(unittest2.TestCase):
         loader = unittest2.TestLoader()
         try:
             loader.loadTestsFromNames(['testcase_1.testfoo'], m)
-        except AttributeError, e:
+        except AttributeError:
+            e = sys.exc_info()[1]
             self.assertEqual(str(e), "type object 'MyTestCase' has no attribute 'testfoo'")
         else:
             self.fail("Failed to raise AttributeError")
@@ -911,12 +919,8 @@ class Test_TestLoader(unittest2.TestCase):
         # We're going to try to load this module as a side-effect, so it
         # better not be loaded before we try.
         #
-        # Why pick audioop? Google shows it isn't used very often, so there's
-        # a good chance that it won't be imported when this test is run
-        module_name = 'audioop'
-
-        if module_name in sys.modules:
-            del sys.modules[module_name]
+        module_name = 'unittest2.test.dummy'
+        sys.modules.pop(module_name, None)
 
         loader = unittest2.TestLoader()
         try:
@@ -925,7 +929,7 @@ class Test_TestLoader(unittest2.TestCase):
             self.assertIsInstance(suite, loader.suiteClass)
             self.assertEqual(list(suite), [unittest2.TestSuite()])
 
-            # audioop should now be loaded, thanks to loadTestsFromName()
+            # module should now be loaded, thanks to loadTestsFromName()
             self.assertIn(module_name, sys.modules)
         finally:
             if module_name in sys.modules:
@@ -1113,7 +1117,7 @@ class Test_TestLoader(unittest2.TestCase):
     # getTestCaseNames() and all the loadTestsFromX() methods"
     def test_sortTestMethodsUsing__loadTestsFromTestCase(self):
         def reversed_cmp(x, y):
-            return -cmp(x, y)
+            return -util.three_way_cmp(x, y)
 
         class Foo(unittest2.TestCase):
             def test_1(self): pass
@@ -1129,7 +1133,7 @@ class Test_TestLoader(unittest2.TestCase):
     # getTestCaseNames() and all the loadTestsFromX() methods"
     def test_sortTestMethodsUsing__loadTestsFromModule(self):
         def reversed_cmp(x, y):
-            return -cmp(x, y)
+            return -util.three_way_cmp(x, y)
 
         m = types.ModuleType('m')
         class Foo(unittest2.TestCase):
@@ -1147,7 +1151,7 @@ class Test_TestLoader(unittest2.TestCase):
     # getTestCaseNames() and all the loadTestsFromX() methods"
     def test_sortTestMethodsUsing__loadTestsFromName(self):
         def reversed_cmp(x, y):
-            return -cmp(x, y)
+            return -util.three_way_cmp(x, y)
 
         m = types.ModuleType('m')
         class Foo(unittest2.TestCase):
@@ -1165,7 +1169,7 @@ class Test_TestLoader(unittest2.TestCase):
     # getTestCaseNames() and all the loadTestsFromX() methods"
     def test_sortTestMethodsUsing__loadTestsFromNames(self):
         def reversed_cmp(x, y):
-            return -cmp(x, y)
+            return -util.three_way_cmp(x, y)
 
         m = types.ModuleType('m')
         class Foo(unittest2.TestCase):
@@ -1185,7 +1189,7 @@ class Test_TestLoader(unittest2.TestCase):
     # Does it actually affect getTestCaseNames()?
     def test_sortTestMethodsUsing__getTestCaseNames(self):
         def reversed_cmp(x, y):
-            return -cmp(x, y)
+            return -util.three_way_cmp(x, y)
 
         class Foo(unittest2.TestCase):
             def test_1(self): pass
@@ -1197,15 +1201,15 @@ class Test_TestLoader(unittest2.TestCase):
         test_names = ['test_2', 'test_1']
         self.assertEqual(loader.getTestCaseNames(Foo), test_names)
 
-    # "The default value is the built-in cmp() function"
+    # "The default value is the built-in util.three_way_cmp() function"
     def test_sortTestMethodsUsing__default_value(self):
         loader = unittest2.TestLoader()
-        self.assertTrue(loader.sortTestMethodsUsing is cmp)
+        self.assertTrue(loader.sortTestMethodsUsing is util.three_way_cmp)
 
     # "it can be set to None to disable the sort."
     #
-    # XXX How is this different from reassigning cmp? Are the tests returned
-    # in a random order or something? This behaviour should die
+    # XXX How is this different from reassigning util.three_way_cmp? Are the
+    # tests returned in a random order or something? This behaviour should die
     def test_sortTestMethodsUsing__None(self):
         class Foo(unittest2.TestCase):
             def test_1(self): pass
