@@ -1,6 +1,8 @@
 import sys
 import textwrap
-from StringIO import StringIO
+import traceback
+
+from six.moves import StringIO
 
 import unittest2
 
@@ -258,7 +260,7 @@ class Test_TestResult(unittest2.TestCase):
                 f_globals = {}
         result = unittest2.TestResult()
         self.assertFalse(result._is_relevant_tb_level(Frame))
-        
+
         Frame.tb_frame.f_globals['__unittest'] = True
         self.assertTrue(result._is_relevant_tb_level(Frame))
 
@@ -283,19 +285,24 @@ class Test_TestResult(unittest2.TestCase):
 
     def testFailFastSetByRunner(self):
         runner = unittest2.TextTestRunner(stream=StringIO(), failfast=True)
+        self.testRan = False
         def test(result):
+            self.testRan = True
             self.assertTrue(result.failfast)
-        result = runner.run(test)
+        runner.run(test)
+        self.assertTrue(self.testRan)
 
 
-    @unittest2.expectedFailure
-    def testWasSuccessfulWhenUnexpectedSuccesses(self):
-        result = unittest2.TestResult()
-        result.unexpectedSuccesses.append(self)
-        
-        self.assertFalse(result.wasSuccessful())
+class MockTraceback(object):
+    @staticmethod
+    def format_exception(*_):
+        return ['A traceback']
 
 
+def restore_traceback():
+    unittest2.result.traceback = traceback
+
+                
 class TestOutputBuffering(unittest2.TestCase):
 
     def setUp(self):
@@ -309,65 +316,65 @@ class TestOutputBuffering(unittest2.TestCase):
     def testBufferOutputOff(self):
         real_out = self._real_out
         real_err = self._real_err
-        
+
         result = unittest2.TestResult()
         self.assertFalse(result.buffer)
-    
+
         self.assertIs(real_out, sys.stdout)
         self.assertIs(real_err, sys.stderr)
-        
+
         result.startTest(self)
-        
+
         self.assertIs(real_out, sys.stdout)
         self.assertIs(real_err, sys.stderr)
 
     def testBufferOutputStartTestAddSuccess(self):
         real_out = self._real_out
         real_err = self._real_err
-        
+
         result = unittest2.TestResult()
         self.assertFalse(result.buffer)
-        
+
         result.buffer = True
-    
+
         self.assertIs(real_out, sys.stdout)
         self.assertIs(real_err, sys.stderr)
-        
+
         result.startTest(self)
-        
+
         self.assertIsNot(real_out, sys.stdout)
         self.assertIsNot(real_err, sys.stderr)
         self.assertIsInstance(sys.stdout, StringIO)
         self.assertIsInstance(sys.stderr, StringIO)
         self.assertIsNot(sys.stdout, sys.stderr)
-        
+
         out_stream = sys.stdout
         err_stream = sys.stderr
-        
+
         result._original_stdout = StringIO()
         result._original_stderr = StringIO()
-        
-        print 'foo'
-        print >> sys.stderr, 'bar'
-        
+
+        print('foo')
+        sys.stderr.write('bar\n')
+
         self.assertEqual(out_stream.getvalue(), 'foo\n')
         self.assertEqual(err_stream.getvalue(), 'bar\n')
-        
+
         self.assertEqual(result._original_stdout.getvalue(), '')
         self.assertEqual(result._original_stderr.getvalue(), '')
-        
+
         result.addSuccess(self)
         result.stopTest(self)
-        
+
         self.assertIs(sys.stdout, result._original_stdout)
         self.assertIs(sys.stderr, result._original_stderr)
-        
+
         self.assertEqual(result._original_stdout.getvalue(), '')
         self.assertEqual(result._original_stderr.getvalue(), '')
-        
+
         self.assertEqual(out_stream.getvalue(), '')
         self.assertEqual(err_stream.getvalue(), '')
-        
+
 
     def getStartedResult(self):
         result = unittest2.TestResult()
@@ -376,29 +383,30 @@ class TestOutputBuffering(unittest2.TestCase):
         return result
 
     def testBufferOutputAddErrorOrFailure(self):
+        unittest2.result.traceback = MockTraceback
+        self.addCleanup(restore_traceback)
+
         for message_attr, add_attr, include_error in [
-            ('errors', 'addError', True), 
+            ('errors', 'addError', True),
             ('failures', 'addFailure', False),
-            ('errors', 'addError', True), 
+            ('errors', 'addError', True),
             ('failures', 'addFailure', False)
         ]:
             result = self.getStartedResult()
-            buffered_out = sys.stdout
-            buffered_err = sys.stderr
             result._original_stderr = StringIO()
             result._original_stdout = StringIO()
-            
-            print >> sys.stdout, 'foo'
+
+            print('foo')
             if include_error:
-                print >> sys.stderr, 'bar'
-            
+                sys.stderr.write('bar\n')
+
             addFunction = getattr(result, add_attr)
             addFunction(self, (None, None, None))
             result.stopTest(self)
-            
+
             result_list = getattr(result, message_attr)
             self.assertEqual(len(result_list), 1)
-            
+
             test, message = result_list[0]
             expectedOutMessage = textwrap.dedent("""
                 Stdout:
@@ -410,14 +418,15 @@ class TestOutputBuffering(unittest2.TestCase):
                 Stderr:
                 bar
             """)
-            expectedFullMessage = 'None\n%s%s' % (expectedOutMessage, expectedErrMessage)
+
+            expectedFullMessage = 'A traceback%s%s' % (expectedOutMessage, expectedErrMessage)
 
             self.assertIs(test, self)
             self.assertEqual(result._original_stdout.getvalue(), expectedOutMessage)
             self.assertEqual(result._original_stderr.getvalue(), expectedErrMessage)
             self.assertMultiLineEqual(message, expectedFullMessage)
-        
-        
+
+
 
 if __name__ == '__main__':
     unittest2.main()
